/****************************************************************************
 *
 * Copyright (c) 2008-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
 
package org.ow2.easywsdl.wsdl.test.util;

import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;

import org.ow2.easywsdl.wsdl.util.InputStreamForker;

/**
 * @author Christophe Deneux - Capgemini Sud
 */
public class InputStreamForkerTest extends TestCase {
    
    private static final String AVAILABLE_FILE = "org/ow2/easywsdl/wsdl/test/util/inputstreamforker/available-file.txt";

    public void testAvailableWithReading() {

        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final byte[] bufferOne = new byte[3];
            int bytesRead = isOne.read(bufferOne);
            TestCase.assertEquals(3, bytesRead);
            TestCase.assertEquals("Hi,", new String(bufferOne));

            int availableOne = isOne.available();
            int availableTwo = isf.getInputStreamTwo().available();

            TestCase.assertEquals(24, availableOne);
            TestCase.assertEquals(27, availableTwo);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    public void testReadFullAtOnce() {

        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final byte[] bufferOne = new byte[1024];
            int bytesReadOne = isOne.read(bufferOne);
            TestCase.assertEquals(27, bytesReadOne);

            TestCase.assertEquals(0, isOne.available());
            TestCase.assertEquals(27, isf.getInputStreamTwo().available());

            final InputStream isTwo = isf.getInputStreamTwo();
            final byte[] bufferTwo = new byte[1024];
            final int bytesReadTwo = isTwo.read(bufferTwo);
            TestCase.assertEquals(27, bytesReadTwo);

            TestCase.assertEquals(0, isOne.available());
            TestCase.assertEquals(0, isf.getInputStreamTwo().available());

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    public void testReadPartial() {

        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final InputStream isTwo = isf.getInputStreamTwo();

            final byte[] bufferOneOne = new byte[10];
            int bytesReadOneOne = isOne.read(bufferOneOne);
            TestCase.assertEquals(10, bytesReadOneOne);
            TestCase.assertEquals(17, isOne.available());

            TestCase.assertEquals(27, isTwo.available());

            final byte[] bufferOneTwo = new byte[10];
            int bytesReadOneTwo = isOne.read(bufferOneTwo);
            TestCase.assertEquals(10, bytesReadOneTwo);
            TestCase.assertEquals(7, isOne.available());

            TestCase.assertEquals(27, isTwo.available());

            final byte[] bufferOneThree = new byte[10];
            int bytesReadOneThree = isOne.read(bufferOneThree);
            TestCase.assertEquals(7, bytesReadOneThree);
            TestCase.assertEquals(0, isOne.available());

            TestCase.assertEquals(27, isTwo.available());
            TestCase.assertEquals("Hi,\n\nMy length is 27 bytes.", new String(
                    bufferOneOne)
                    + new String(bufferOneTwo)
                    + new String(bufferOneThree, 0, bytesReadOneThree));

            final byte[] bufferTwoOne = new byte[15];
            int bytesReadTwoOne = isTwo.read(bufferTwoOne);
            TestCase.assertEquals(15, bytesReadTwoOne);
            TestCase.assertEquals(12, isTwo.available());

            final byte[] bufferTwoTwo = new byte[15];
            int bytesReadTwoTwo = isTwo.read(bufferTwoTwo);
            TestCase.assertEquals(12, bytesReadTwoTwo);
            TestCase.assertEquals(0, isTwo.available());

            TestCase.assertEquals("Hi,\n\nMy length is 27 bytes.", new String(
                    bufferTwoOne)
                    + new String(bufferTwoTwo, 0, bytesReadTwoTwo));
            
            final byte[] bufferTwoThree = new byte[150];
            int bytesReadTwoThree = isTwo.read(bufferTwoThree);
            TestCase.assertEquals("EOF must be reached", -1, bytesReadTwoThree);

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    public void testPartialReadAndClose() {

        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final InputStream isTwo = isf.getInputStreamTwo();

            final byte[] bufferOneOne = new byte[10];
            int bytesReadOneOne = isOne.read(bufferOneOne);
            TestCase.assertEquals(10, bytesReadOneOne);
            TestCase.assertEquals(17, isOne.available());

            TestCase.assertEquals(27, isTwo.available());

            final byte[] bufferOneTwo = new byte[10];
            int bytesReadOneTwo = isOne.read(bufferOneTwo);
            TestCase.assertEquals(10, bytesReadOneTwo);
            TestCase.assertEquals(7, isOne.available());
            
            isOne.close();
            // As the forked input stream 'isOne' is closed, available bytes of
            // the orihinal input stream are no more available for this forked
            // input stream.
            TestCase.assertEquals(0, isOne.available());

            TestCase.assertEquals(27, isTwo.available());

            TestCase.assertEquals(27, isTwo.available());
            TestCase.assertEquals("Hi,\n\nMy length is 27", new String(
                    bufferOneOne)
                    + new String(bufferOneTwo));

            final byte[] bufferTwoOne = new byte[15];
            int bytesReadTwoOne = isTwo.read(bufferTwoOne);
            TestCase.assertEquals(15, bytesReadTwoOne);
            TestCase.assertEquals(12, isTwo.available());

            final byte[] bufferTwoTwo = new byte[15];
            int bytesReadTwoTwo = isTwo.read(bufferTwoTwo);
            TestCase.assertEquals(12, bytesReadTwoTwo);
            TestCase.assertEquals(0, isTwo.available());

            TestCase.assertEquals("Hi,\n\nMy length is 27 bytes.", new String(
                    bufferTwoOne)
                    + new String(bufferTwoTwo, 0, bytesReadTwoTwo));

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }

    }

    public void testClosedInputStreamNoException() {
        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final InputStream isTwo = isf.getInputStreamTwo();

            final byte[] bufferOneOne = new byte[1024];
            int bytesReadOneOne = isOne.read(bufferOneOne);
            TestCase.assertEquals(27, bytesReadOneOne);
            TestCase.assertEquals(0, isOne.available());
            TestCase.assertEquals(27, isTwo.available());
            TestCase.assertEquals("Hi,\n\nMy length is 27 bytes.", new String(
                    bufferOneOne, 0, bytesReadOneOne));

            isOriginal.close();

            final byte[] bufferTwoOne = new byte[27];
            int bytesReadTwoOne = isTwo.read(bufferTwoOne);
            TestCase.assertEquals(27, bytesReadTwoOne);
            try {
                TestCase.assertEquals(0, isTwo.available());
                TestCase.assertEquals("Not thrown IOException", true, false);
            } catch (final IOException e) {
                // NOP. It is the expected case.
            }
            TestCase.assertEquals("Hi,\n\nMy length is 27 bytes.", new String(
                    bufferTwoOne, 0, bytesReadTwoOne));

        } catch (IOException e) {
            e.printStackTrace();
            TestCase.fail();
        }
    }
    
    public void testClosedInputStreamWithException() {
        try {
            final InputStream isOriginal = Thread
                    .currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(AVAILABLE_FILE);

            final InputStreamForker isf = new InputStreamForker(isOriginal);

            final InputStream isOne = isf.getInputStreamOne();
            final InputStream isTwo = isf.getInputStreamTwo();

            final byte[] bufferOneOne = new byte[10];
            int bytesReadOneOne = isOne.read(bufferOneOne);
            TestCase.assertEquals(10, bytesReadOneOne);
            TestCase.assertEquals(17, isOne.available());
            TestCase.assertEquals(27, isTwo.available());
            TestCase.assertEquals("Hi,\n\nMy le", new String(
                    bufferOneOne, 0, bytesReadOneOne));

            isOriginal.close();

            final byte[] bufferTwoOne = new byte[1024];
            isTwo.read(bufferTwoOne);
            TestCase.assertEquals("Not thrown IOException", true, false);

        } catch (IOException e) {
            // NOP. It is the expected case. A IOEXception is thrown when calling read of the second forked stream
        }
    }

}
